# -*- coding: utf-8 -*-

# Automatically generated - don't edit.
# Use `python setup.py build_ui` to update it.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ScriptDetails(object):
    def setupUi(self, ScriptDetails):
        ScriptDetails.setObjectName("ScriptDetails")
        ScriptDetails.setWindowModality(QtCore.Qt.WindowModal)
        ScriptDetails.resize(700, 284)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(ScriptDetails.sizePolicy().hasHeightForWidth())
        ScriptDetails.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(ScriptDetails)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.script_version = QtWidgets.QLineEdit(ScriptDetails)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.script_version.sizePolicy().hasHeightForWidth())
        self.script_version.setSizePolicy(sizePolicy)
        self.script_version.setMinimumSize(QtCore.QSize(100, 0))
        self.script_version.setMaximumSize(QtCore.QSize(100, 16777215))
        self.script_version.setBaseSize(QtCore.QSize(0, 0))
        self.script_version.setObjectName("script_version")
        self.horizontalLayout.addWidget(self.script_version)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.label_4 = QtWidgets.QLabel(ScriptDetails)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout.addWidget(self.label_4)
        self.script_last_updated = QtWidgets.QLineEdit(ScriptDetails)
        self.script_last_updated.setMinimumSize(QtCore.QSize(200, 0))
        self.script_last_updated.setMaximumSize(QtCore.QSize(200, 16777215))
        self.script_last_updated.setObjectName("script_last_updated")
        self.horizontalLayout.addWidget(self.script_last_updated)
        self.last_updated_now = QtWidgets.QPushButton(ScriptDetails)
        self.last_updated_now.setObjectName("last_updated_now")
        self.horizontalLayout.addWidget(self.last_updated_now)
        self.gridLayout.addLayout(self.horizontalLayout, 2, 2, 1, 1)
        self.label_2 = QtWidgets.QLabel(ScriptDetails)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.label_3 = QtWidgets.QLabel(ScriptDetails)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 2, 0, 1, 1)
        self.label_5 = QtWidgets.QLabel(ScriptDetails)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 3, 0, 1, 1)
        self.label = QtWidgets.QLabel(ScriptDetails)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.script_license = QtWidgets.QLineEdit(ScriptDetails)
        self.script_license.setObjectName("script_license")
        self.gridLayout.addWidget(self.script_license, 3, 2, 1, 1)
        self.label_6 = QtWidgets.QLabel(ScriptDetails)
        self.label_6.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 4, 0, 1, 1)
        self.script_description = QtWidgets.QPlainTextEdit(ScriptDetails)
        self.script_description.setObjectName("script_description")
        self.gridLayout.addWidget(self.script_description, 4, 2, 1, 1)
        self.script_author = QtWidgets.QLineEdit(ScriptDetails)
        self.script_author.setObjectName("script_author")
        self.gridLayout.addWidget(self.script_author, 1, 2, 1, 1)
        self.script_title = QtWidgets.QLineEdit(ScriptDetails)
        self.script_title.setObjectName("script_title")
        self.gridLayout.addWidget(self.script_title, 0, 2, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(ScriptDetails)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Close|QtWidgets.QDialogButtonBox.Save)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(ScriptDetails)
        QtCore.QMetaObject.connectSlotsByName(ScriptDetails)

    def retranslateUi(self, ScriptDetails):
        _translate = QtCore.QCoreApplication.translate
        ScriptDetails.setWindowTitle(_("File Naming Script Metadata"))
        self.script_version.setToolTip(_("Version number of the file naming script."))
        self.label_4.setText(_("Last Updated:"))
        self.script_last_updated.setToolTip(_("Date and time the file naming script was last updated (UTC)."))
        self.last_updated_now.setText(_("Now"))
        self.label_2.setText(_("Author:"))
        self.label_3.setText(_("Version:"))
        self.label_5.setText(_("License:"))
        self.label.setText(_("Title:"))
        self.script_license.setToolTip(_("License under which the file naming script is available."))
        self.label_6.setText(_("Description:"))
        self.script_description.setToolTip(_("Brief description of the file naming script, including any required plugins."))
        self.script_author.setToolTip(_("The author of the file naming script."))
